#ifndef CCPYTHIA_HH
#define CCPYTHIA_HH

#include "AGILe/Generator.hh"
#include "Pythia8/Pythia.h"
#include "Pythia8/HepMCInterface.h"

namespace AGILe {

  class CCPythia : public Generator {
  public:
    /// Standard constructor.
    CCPythia();

    /// Destructor.
    ~CCPythia() { } 

    /// Set up initial state from supplied params.
    Status setGenSpecificInitialState(PdgCode p1 = PROTON, double e1 = 7000.0, 
                                      PdgCode p2 = PROTON, double e2 = 7000.0);

    // Set random seed before initialization.
    virtual Status setSeed(const int seed);
    
    /// Run the generator for one event.
    Status makeEvent(HepMC::GenEvent& evt);

    /// Make sure that generic setParam is visible.
    using Generator::setParam;
    
    /// Pass a parameter to the generator.
    Status setParam(const string& name, const string& value);

    /// Convert bool arguments to "on"/"off".
    Status setParam(const string& name, const bool& value) {
      const string boolval = (value) ? "on" : "off";
      return setParam(name, boolval);
    }

    /// Get the cross section as determined by Pythia8
    const double getCrossSection();
    
    /// Tidy up after ourselves.
    Status finalize();

  protected:
    /// Set up default params etc.
    Status initialize();

    /// Fill a HepMC event.
    Status fillEvent(HepMC::GenEvent& evt);

    // Pythia native generator.
    Pythia8::Pythia _pythia;

    /// Pythia 8 to HepMc converter
    HepMC::I_Pythia8 _toHepMC;
    
  private:

    /// Internal storage of initial state params
    string _particleName1;
    string _particleName2;
    double _cmsMomentum;
    PdgCode _p1, _p2;

  };

}
#endif
