#ifndef HERWIG_WRAPPER65_HH
#define HERWIG_WRAPPER65_HH

#include <ctype.h>
#include "AGILe/FHerwig/FHerwigWrapper.hh"

/// Herwig common block declarations
/// Adapted from HepMC wrapper for Herwig by Matt Dobbs


extern "C" {

  // COMMON/HWPROC/EBEAM1,EBEAM2,PBEAM1,PBEAM2,IPROC,MAXEV
  #define FC_HWPROC FC_FUNC(hwproc, HWPROC)
  extern struct {
    double EBEAM1,EBEAM2,PBEAM1,PBEAM2;
    int IPROC,MAXEV;
  } FC_HWPROC;


  // CHARACTER*8 PART1,PART2
  // COMMON/HWBMCH/PART1,PART2
  #define FC_HWBMCH FC_FUNC(hwbmch, HWBMCH)
  extern struct {
    char PART1[8],PART2[8];
    
  } FC_HWBMCH;

  // INTEGER IPART1,IPART2
  // COMMON/HWBEAM/IPART1,IPART2
  #define FC_HWBEAM FC_FUNC(hwbeam, HWBEAM)
  extern struct {
    int IPART1,IPART2;
    
  } FC_HWBEAM;


  // COMMON/HWEVNT/AVWGT,EVWGT,GAMWT,TLOUT,WBIGST,WGTMAX,WGTSUM,WSQSUM,
  // & IDHW(NMXHEP),IERROR,ISTAT,LWEVT,MAXER,MAXPR,NOWGT,NRN(2),NUMER,
  // & NUMERU,NWGTS,GENSOF
  const int herwig_hepevt_size = 10000;
  #define FC_HWEVNT FC_FUNC(hwevnt, HWEVNT)
  extern struct {
    double AVWGT,EVWGT,GAMWT,TLOUT,WBIGST,WGTMAX,WGTSUM,WSQSUM;
    int IDHW[herwig_hepevt_size],IERROR,ISTAT,LWEVT,MAXER,MAXPR;
    int NOWGT,NRN[2],NUMER,NUMERU,NWGTS;
    int GENSOF; // Beware! in F77 this is logical
  } FC_HWEVNT;


  // C Basic parameters (and quantities derived from them)
  // COMMON/HWPRAM/AFCH(16,2),ALPHEM,B1LIM,BETAF,BTCLM,CAFAC,CFFAC,
  // & CLMAX,CLPOW,CLSMR(2),CSPEED,ENSOF,ETAMIX,F0MIX,F1MIX,F2MIX,GAMH,
  // & GAMW,GAMZ,GAMZP,GEV2NB,H1MIX,PDIQK,PGSMX,PGSPL(4),PHIMIX,PIFAC,
  // & PRSOF,PSPLT(2),PTRMS,PXRMS,QCDL3,QCDL5,QCDLAM,QDIQK,QFCH(16),QG,
  // & QSPAC,QV,SCABI,SWEIN,TMTOP,VFCH(16,2),VCKM(3,3),VGCUT,VQCUT,
  // & VPCUT,ZBINM,EFFMIN,OMHMIX,ET2MIX,PH3MIX,GCUTME,
  // & IOPREM,IPRINT,ISPAC,LRSUD,LWSUD,MODPDF(2),NBTRY,NCOLO,NCTRY,
  // & NDTRY,NETRY,NFLAV,NGSPL,NSTRU,NSTRY,NZBIN,IOP4JT(2),NPRFMT,
  // & AZSOFT,AZSPIN,CLDIR(2),HARDME,NOSPAC,PRNDEC,PRVTX,SOFTME,ZPRIME,
  // & PRNDEF,PRNTEX,PRNWEB
  #define FC_HWPRAM FC_FUNC(hwpram, HWPRAM)
  extern struct {
    double AFCH[2][16],ALPHEM,B1LIM,BETAF,BTCLM,CAFAC,CFFAC,
      CLMAX,CLPOW,CLSMR[2],CSPEED,ENSOF,ETAMIX,F0MIX,F1MIX,F2MIX,GAMH,
      GAMW,GAMZ,GAMZP,GEV2NB,H1MIX,PDIQK,PGSMX,PGSPL[4],PHIMIX,PIFAC,
      PRSOF,PSPLT[2],PTRMS,PXRMS,QCDL3,QCDL5,QCDLAM,QDIQK,QFCH[16],QG,
      QSPAC,QV,SCABI,SWEIN,TMTOP,VFCH[2][16],VCKM[3][3],VGCUT,VQCUT,
      VPCUT,ZBINM,EFFMIN,OMHMIX,ET2MIX,PH3MIX,GCUTME;
    int IOPREM,IPRINT,ISPAC,LRSUD,LWSUD,MODPDF[2],NBTRY,NCOLO,NCTRY,
      NDTRY,NETRY,NFLAV,NGSPL,NSTRU,NSTRY,NZBIN,IOP4JT[2],NPRFMT;
    int AZSOFT,AZSPIN,CLDIR[2],HARDME,NOSPAC,PRNDEC,PRVTX,SOFTME,
      ZPRIME,PRNDEF,PRNTEX,PRNWEB; // In F77 these are logical
  } FC_HWPRAM;


  //      COMMON/HWPRCH/AUTPDF(2),BDECAY
  #define FC_HWPRCH FC_FUNC(hwprch, HWPRCH)
  extern struct {
    char AUTPDF[2][20],BDECAY;
  } FC_HWPRCH;


  //      COMMON /HW6506/PDFX0,PDFPOW
  #define FC_HW6506 FC_FUNC(hw6506, HW6506)
  extern struct {
    double PDFX0,PDFPOW;
  } FC_HW6506;


  //      COMMON/HWUCLU/CLDKWT(NMXCDK),CTHRPW(12,12),PRECO,RESN(12,12),
  //     & RMIN(12,12),LOCN(12,12),NCLDK(NMXCDK),NRECO,CLRECO
  const int nmxcdk = 4000;
  #define FC_HWUCLU FC_FUNC(hwuclu, HWUCLU)
  extern struct {
    double CLDKWT[nmxcdk],CTHRPW[12][12],PRECO,RESN[12][12],RMIN[12][12];
    int LOC[12][12],NCLDK[nmxcdk],NRECO,CLRECO;
    // CLRECO is F77 logical.
  } FC_HWUCLU;


  //      COMMON/HWHARD/ASFIXD,CLQ(7,6),COSS,COSTH,CTMAX,DISF(13,2),EMLST,
  //    & EMMAX,EMMIN,EMPOW,EMSCA,EPOLN(3),GCOEF(7),GPOLN,OMEGA0,PHOMAS,
  //     & PPOLN(3),PTMAX,PTMIN,PTPOW,Q2MAX,Q2MIN,Q2POW,Q2WWMN,Q2WWMX,QLIM,
  //     & SINS,THMAX,Y4JT,TMNISR,TQWT,XX(2),XLMIN,XXMIN,YBMAX,YBMIN,YJMAX,
  //     & YJMIN,YWWMAX,YWWMIN,WHMIN,ZJMAX,ZMXISR,IAPHIG,IBRN(2),IBSH,
  //     & ICO(10),IDCMF,IDN(10),IFLMAX,IFLMIN,IHPRO,IPRO,MAPQ(6),MAXFL,
  //     & BGSHAT,COLISR,FSTEVT,FSTWGT,GENEV,HVFCEN,TPOL,DURHAM
  #define FC_HWHARD FC_FUNC(hwhard, HWHARD)
  extern struct {
    double ASFIXD,CLQ[6][7],COSS,COSTH,CTMAX,DISF[2][13],EMLST,EMMAX,
      EMMIN,EMPOW,EMSCA,EPOLN[3],GCOEF[7],GPOLN,OMEGA0,PHOMAS,PPOLN[3],
      PTMAX,PTMIN,PTPOW,Q2MAX,Q2MIN,Q2POW,Q2WWMN,Q2WWMX,QLIM,
      SINS,THMAX,Y4JT,TMNISR,TQWT,XX[2],XLMIN,XXMIN,YBMAX,YBMIN,YJMAX,
      YJMIN,YWWMAX,YWWMIN,WHMIN,ZJMAX,ZMXISR;
    int IAPHIG,IBRN[2],IBSH,ICO[10],IDCMF,IDN[10],IFLMAX,IFLMIN,IHPRO,
      IPRO,MAPQ[6],MAXFL,BGSHAT,COLISR,FSTEVT,FSTWGT,GENEV,HVFCEN,TPOL,DURHAM;
  } FC_HWHARD;


  // HERWIG block data initializer: call "hwudat();" before starting
  #define FC_HWPUDAT FC_FUNC(hwudat, HWUDAT)
  void FC_HWUDAT(void);

}

#endif
